### Basic example
Insert the following script anywhere in the document 
```javascript
<script>
    (function (w,d,s,o,f,js,fjs) {
        w[o] = w[o] || function () { (w[o].q = w[o].q || []).push(arguments) };
        js = d.createElement(s); fjs = d.getElementsByTagName(s)[0];
        js.id = o; js.src = f; js.async = true; fjs.parentNode.insertBefore(js, fjs);
    }(window, document, 'script', 'eyeZon', 'https://storage.googleapis.com/eyezonscriptv2/static/js/eyezonwidget.js'));

    eyeZon('init', {
        businessId: 'your-business-id',
        region: 'ru-1'
    });
</script>
```
    
    
Add **data-eyezon** attribute to an element
```html
<button data-eyezon>Start eyezon chat</button>
```
    
By default, chats are uniquely identified by current URL without query string and hash. 
So https://domain.com, https://domain.com/product, http://domain.com/product will all have chats of their own. 

Chat titles are pulled from the first H1 element on page.

Chat settings are loaded for the first button listed in admin area. 

#### Unsupported browsers
Only ES-6 compliant browsers are supported. System alert will be shown if trying 
to start a dialog in an outdated browser (e.g. IE-11). To automatically remove 
blocks containing eyezon buttons on unsupported browsers 
**data-eyezon-removable** attribute can be used. 
```html
<div data-eyezon-removable>
    This block will get removed in IE 11 and other outdated browsers
    <button data-eyezon>Start</button>
</div>
```

### Chat customization
Attributes **data-eyezon-target** and **data-eyezon-title** can be used to specify chat target and title. 
To specify a custom referrer URL **data-eyezon-product-url** can be used, otherwise the current URL up to the pathname is used. 

For example, if you insert the following button on different pages, it will always open the same chat titled "TV & Home Theaters": 
```html
<button data-eyezon
        data-eyezon-target="Electronics Department"
        data-eyezon-title="TV & Home Theaters"
        data-eyezon-product-url="https://domain.com/electronics">
    Start eyezon chat
</button>
```
    
    
Or if you have multiple buttons on the same page and want them to open different chats:
```html
<button data-eyezon
        data-eyezon-target="SKU-1"
        data-eyezon-title="Samsung Television">
    Show me Samsung TV set
</button>
<button data-eyezon
        data-eyezon-target="SKU-2"
        data-eyezon-title="Sony Television">
    Show me Sony TV set
</button>
```
    
    
Attribute **data-eyezon-auto-submit** can be added to automatically submit the specified message. Won't work if there are no available streamers and the client isn't subscribed to notifications. 
```html
<button data-eyezon data-eyezon-auto-submit="Automatically submitted message">
    Show me Sony TV set
</button>
```
    
    
### Settings
Chats can have different settings: UI, texts and streamers. These settings are specific to
a button and are identified by buttonId. Settings are cached for 5 minutes in session storage (i.e. per browser tab). 

As an example, a company identified by businessId 5e81cf421cb513002ed6f39e has TV-set and Fridge department, 
wants them to have different color schemes, welcome texts and different specialists to process streaming 
requests: 
```javascript
eyeZon('init', {
    businessId: '5e81cf421cb513002ed6f39e'
});
```
    

In this case the company creates 2 buttons for respective departments in the admin area. 
If a client is looking at a TV set, the relevant TV department buttonId 
can be set with **data-eyezon** attribute:
```html
<button data-eyezon="5e81cf891cb513002ed6f3ab"
        data-eyezon-target="SKU-1"
        data-eyezon-title="Samsung Television">
    Show me Samsung TV-Set
</button>
```
    

For a fridge they would also specify a corresponding buttonId:
```html
<button data-eyezon="5e81cf891cb513002ed6f3ac"
        data-eyezon-target="SKU-3"
        data-eyezon-title="LG Refrigerator">
    Show me LG Refrigerator
</button>
```
    

### Advanced routing
By default, dialogs are sent to all available streamers assigned to the button. 
To send dialogs to specific streamers **data-eyezon-streamer-ids** can be used. To send dialogs to specific shops **data-eyezon-shop-ids** can be used. Streamers still have to be assigned to the specified button.
The attribute expects an array of strings and is parsed with JSON.parse. Non-arrays are ignored. Streamer and shop ids refer to the ids in the customer system. 
```html
<button data-eyezon="5e81cf891cb513002ed6f3ac"
        data-eyezon-streamer-ids='["streamer1", "streamer2"]'>
    Send to specific streamers
</button>

<button data-eyezon="5e81cf891cb513002ed6f3ac"
        data-eyezon-shop-ids='["shop1", "shop2"]'>
    Send to specific shops
</button>
```

There is also a **resetRoutingAfterSeconds** [init param](#parameter-overview). If the dialog is open and no streamer has responded within the specified period, it will reset streamerIds and shopIds, and resend the request to all available streamers of the current button.

#### Two-layer routing
You can also add two-level routing. 
Combine the data-eyezon-streamer-ids and data-eyezon-second-shop-ids parameters to send dialogs to streamers first and then to shops.
Combine the data-eyezon-shop-ids and data-eyezon-second-streamer-ids parameters to send dialogs to shops first and then to streamers.

```html
<button data-eyezon="5e81cf891cb513002ed6f3ac"
        data-eyezon-streamer-ids='["streamer1", "streamer2"]'
        data-eyezon-second-shop-ids='["shop1", "shop2"]'>
    Send to specific streamers then shops
</button>

<button data-eyezon="5e81cf891cb513002ed6f3ac"
        data-eyezon-shop-ids='["shop1", "shop2"]'
        data-eyezon-second-streamer-ids='["streamer1", "streamer2"]'>
    Send to specific shops then streamers
</button>
```

There is also a **secondRoutingAfterSeconds** [init param](#parameter-overview).
If the dialog is open and no streamer has responded within the specified *secondRoutingAfterSeconds* period, then resend the request to *data-eyezon-second-shop-ids* or *data-eyezon-second-streamer-ids*. If no streamer has responded within the specified 'resetRoutingAfterSeconds' period again, it will reset streamerIds and shopIds, and resend the request to all available streamers of the current button.
    
### Parameter overview
```javascript
eyeZon('init', {
    businessId: '5e81cf421cb513002ed6f39e', 
    region: 'ru-1',
    live: false,
    position: 'BOTTOM_RIGHT',
    language: 'RU',
    fallbackLanguage: 'RU',
    hideEyeAfterMinutes: 24 * 60,
    hideInputIfPossible: false,
    logging: false,
    unclosable: false,
    streamingBackend: null,
    recordsUrl: null,
    recurIntroVideo: false,
    secondRoutingAfterSeconds: false,
    resetRoutingAfterSeconds: false,
    beforeRespondedVideo: null,
    beforeRespondedIframe: null,
});
```
    
* businessId - required, used to check active subscriptions and set buttonIds. Also accepts in array of business ids. 
* region - data center location. Currently available regions are 'ru-1', 'us-1' and 'eu-1'. Defaults to 'ru-1'
* live - use event delegation to watch for dynamically added triggers. Defaults to false. 
* position - overrides position settings fetched from server 
* language - if not specified, tries lang attribute from HTML element, if not found - uses navigator.language 
* fallbackLanguage - if translations for automatically determined language don't exist falls back to the provided language, won't be used if the language param is specified. Defaults to "RU" on "ru-1" region, "EN" on all the others.
* hideEyeAfterMinutes - how long it takes in minutes to hide the eye after all messages have been read, defaults to 24 * 60. 
* hideInputIfPossible - hides message input if predefined questions are provided for the current locale and there is at least one streamer online, defaults to false
* logging - whether to output errors and events to browser console (will also log if window.eyezon_TurnOnLogging is truthy), defaults to false
* unclosable - will make it impossible to close the chat wrapper once it is open (e.g. to use in WebView of a mobile app), defaults to false
* streamingBackend - overrides region specific url of the streaming server (e.g. "wss://wsstest.eyezon.online")
* recordsUrl - overrides region specific url of the stream recordings server (e.g. "https://recordstest.eyezon.online")
* recurIntroVideo - whether to keep displaying intro video even if the client has already had a dialog, defaults to false
* secondRoutingAfterSeconds - used in advanced routing with two-layer [advanced routing](#advanced-routing). Defaults to false. 
* resetRoutingAfterSeconds - used in advanced routing [advanced routing](#advanced-routing). Defaults to false. 
* beforeRespondedVideo - URL of a video shown after a client sends a request. The video is automatically closed when a streamer responds. If the user closes it manually, it will never be shown again. 
* beforeRespondedIframe - URL of an iframe shown after a client sends a request. The iframe is automatically closed when a streamer responds. If the user closes it manually, it will never be shown again. The iframe isn't sandboxed.

### Passing multiple business ids
If **businessId** parameter is specified as an array, you must always provide a button id in **data-eyezon** attribute and API methods. 
The widget will work if at least one of the businesses has active subscriptions. But trying to open a chat routed to an expired business won't work.  

```javascript
eyeZon('init', {
    businessId: ['5e81cf421cb513002ed6f39e', '507f191e810c19729de860ea']
});
```

### Aspect ratio
The video aspect ratio is 3:4. If you provide an intro video, it has to conform to that ratio 
to avoid blank fields around the player.

### Important notice

The page the widget is installed on must be served over HTTPS. If it isn't, WebRTC and microphone will be blocked by the browser. 
The widget will fall back to software decoding and drawing the output on Canvas, which is very inefficient. 

### Url triggers 
Adding eyezon query string param to a URL can be used to open chats: 

**https://domain.com/?eyezon** - will open a chat routed to first buttonId, targeted at current URL without query string, and with title taken from the first H1 element

It is also possible to specify target, title, buttonId, autoSubmit, putInCart, productUrl, openChatList, streamerIds and shopIds with the corresponding query string params:

**https://www.domain.com/?eyezon&target=SKU-1&title=Samsung%20Television&buttonId=5e81cf891cb513002ed6f3ab&autoSubmit=Show%20me&putInCart=true&productUrl=https%3A%2F%2Fwww.domain.com%2Fcustom-referrer-url&streamerIds=[%22streamer1%22,%20%22streamer2%22]**

Note that streamerIds and shopIds are JSON stringified arrays.

openChatList param will open widget with dialog history mode

### API
The widget injects eyeZon in global namespace. 
Initial calls are queued until after the initialization. 

All methods are async and return a promise. Every call is put onto the same promise chain and is run sequentially.  

#### setLanguage
```javascript
eyeZon('setLanguage', 'RU');
```
    
If the server doesn't provide translations for the specified language, it will default to fallbackLanguage

#### setPosition
```typescript
eyeZon('setPosition', 'BOTTOM_CENTER');

enum Position {
  BOTTOM_LEFT = 'BOTTOM_LEFT',
  BOTTOM_CENTER = 'BOTTOM_CENTER',
  BOTTOM_RIGHT = 'BOTTOM_RIGHT',
  MID_LEFT = 'MID_LEFT',
  MID_CENTER = 'MID_CENTER',
  MID_RIGHT = 'MID_RIGHT'
}
```
    
If provided position is erroneous will default to **BOTTOM_RIGHT**

#### open
```javascript
eyeZon('open', { 
    target: 'Test', 
    title: 'Test', 
    buttonId: '5e81cf891cb513002ed6f3ab',
    autoSubmit: 'Test message',
    putInCart: false,
    productUrl: "https://domain.com/custom-referrer-url",
    streamerIds: [],
    shopIds: []
});
```
    
* target - unique chat intent identifier. If left out defaults to current URL without query string and hash. Not displayed in the UI. Corresponds to data-eyezon-target.
* title - chat title, defaults to textContent of first H1 in the document. Both client and streamer can see it. Corresponds to data-eyezon-title.
* buttonId - used for loading settings and routing to appropriate streamers, defaults to first buttonId in admin area. Corresponds to data-eyezon.
* autoSubmit - if possible, immediately creates a new dialog and sends specified text as first message. Corresponds to data-eyezon-auto-submit.
* putInCart - will add "Add to cart" button in the UI if STREAMER_PUT_IN_CART_ACCEPTED event is being listened to and the target is provided. Corresponds to data-eyezon-put-in-cart.
* productUrl - overrides the referrer URL, defaults to the current URL up to the pathname - *window.location.href.split("?")[0]*. 
  Used in statistics reports and for opening the origin webpage when clicking on "Product page" in mobile apps. Corresponds to data-eyezon-product-url.
* streamerIds - used in [advanced routing](#advanced-routing). Non-arrays are ignored. Corresponds to data-eyezon-streamer-ids.  
* shopIds - used in [advanced routing](#advanced-routing). Non-arrays are ignored. Corresponds to data-eyezon-shop-ids.  

#### setDefaultClientField
Pre-fills *name*, *phone* or *email* client fields with default data if button settings 
have them enabled. Once called, stores the data in localStorage **indefinitely**. 
Can be called before *init*. Doesn't change existing client data. 
```javascript
eyeZon('setDefaultClientField', 'name', 'John Doe');  
eyeZon('setDefaultClientField', 'phone', '+7 (123) 123-12-12'); 
eyeZon('setDefaultClientField', 'phoneCountry', 'ru'); // ISO 3166-1 alpha-2 code
eyeZon('setDefaultClientField', 'email', 'test@domain.com');

// It is also possible to remove prefilled data, e.g. if the client revoked his consent
eyeZon('setDefaultClientField', 'name', ''); 
```

#### setClientField
Sets *name*, *phone* or *email* client fields and checks 
*agree with privacy terms* checkbox. Must be called after *init*.
**The client must have agreed to privacy terms** before this method
is called. The set fields will only be stored in local storage 
(i.e. between requests) if there is a dialog in history. 
```javascript
eyeZon('setClientField', 'name', 'John Doe');  
eyeZon('setClientField', 'phone', '+7 (123) 123-12-12'); 
eyeZon('setClientField', 'phoneCountry', 'ru'); // ISO 3166-1 alpha-2 code
eyeZon('setClientField', 'email', 'test@domain.com');

// Also possible to clear user data, e.g. if the client revoked his consent
eyeZon('setClientField', 'name', ''); 
```


#### reset
Unsets dialogs, user data, notification permissions and clientId 
for the region specified in init. 
Turns off settings cache for the lifetime of the current page.

If called before the widget code is loaded - will be hoisted to the 
beginning of the promise chain.

If the method is called while the widget is open -
will kick the user out of the stream\chat session, close the widget,
and re-fetch subscriptions and buttonIds.


```javascript
eyeZon('reset')
```

Reset can also be run by appending **eyezonReset** query param to the URL.

**hideEyeAfterMinutes parameter should be preferred** over the
reset method in most scenarios.

### Event system

#### on
```javascript
eyeZon('on', type, callback)
```
    
 * type - eyezon event to notify of  
 * callback - function with 1 parameter  

#### off
```javascript
eyeZon('on', type, callback)
```
    
##### Available event types
All events receive **title**, **target**, **buttonId** params. 

* BUTTON_CLICKED - emitted per every successful opening of the widget (e.g. clicking on a button with data-eyezon attribute, opening from URL or by an API)
* DIALOG_CREATED - emitted per every creation of a new dialog; adds **dialogId** param. Adds **predefinedMessageIndex** and **predefinedMessage** params if dialog was started by clicking on a predefined question. Adds **message** param if it was started by typing in a message.  
* PASSIVE_REQUEST - emitted per creation of a new dialog when streamers aren't available; adds **dialogId** param
* MISSED_REQUEST - emitted when the timer runs out; adds **dialogId** param
* STREAM_JOINED - emitted every time the client enters a stream; adds **dialogId** param
* CHAT_JOINED - emitted regardless of dialog existence and status or how the client navigated there (same as BUTTON_CLICKED plus opened from the dialog list within the widget UI)
* DIALOG_CLOSED - only when the dialog got closed by the client; adds **dialogId** param
* CHAT_OVERLAY_CLOSED - emitted after clicking on the overlay background or close icon. Also emitted on beforeunload window event when the widget is open (beforeunload doesn't fire in certain scenarios, e.g. when the system kills a minimized browser app on a mobile device)
* CLOSE_CHAT_BUTTON_CLICKED - emitted after clicking on the close chat button only. 
* AUDIO_SENT - emitted every time the client sends an audio message outside of stream; adds **dialogId** param
* VIDEO_CLICKED - emitted every time the client clicks on a regular video; adds **dialogId** param
* STREAM_REPLAY_CLICKED - emitted every time the client clicks on a stream replay video; adds **dialogId** param
* PRODUCT_PAGE_CLICKED - emitted every time the client clicks on a product page link. The link appears in 
top right corner if the client isn't on the same url the chat was open on (minus the query string). 
Adds **dialogId** and **url** params
* DIALOG_RATED - emitted when the client clicks on a star. Adds **dialogId** and **rating** params   
* DIALOG_MARKED_UNRATED - emitted when the client clicks on "Close without rating"; adds **dialogId**   
* GAME_OPENED 
* STREAM_MIC_ON - emitted after the client successfully starts audio stream; adds **dialogId** param  
* EMAIL_NOTIFICATIONS_ON - adds **dialogId** param if the dialog exists (not on a "No Streamers Available" screen) 
* SMS_NOTIFICATIONS_ON - adds **dialogId** param if the dialog exists (not on a "No Streamers Available" screen) 
* PUSH_NOTIFICATIONS_ON - adds **dialogId** param if the dialog exists
* INTRO_VIDEO_CLICKED - emitted when the client unmutes the intro video 
* BEFORE_RESPONDED_VIDEO_CLICKED - emitted when the client unmutes the video set with beforeRespondedVideo init parameter
* CLIENT_FIRST_REPLIED - emitted per second message from the client per dialog (i.e. first message after the dialog was accepted by the streamer); adds **message** param
* PUT_IN_CART - initiated by an API call in the host page, see [Emitting analytics events](#emitting-analytics-events) section for details
* PRODUCT_SOLD_BY_STREAMER - emitted after the host page initiates a CART_SUBMITTED event, fires per every product sold by a streamer, see [Emitting analytics events](#emitting-analytics-events) section for details
* PRODUCT_SOLD - emitted after CART_SUBMITTED per every product **possibly** sold by a streamer, see [Emitting analytics events](#emitting-analytics-events) section for details
* STREAMER_PUT_IN_CART_ACCEPTED - emitted when the client accepts "Would you like to put this product in cart" window, see ["Put in cart suggestion"](#put-in-cart-suggestion)  
* STREAMER_PUT_IN_CART_DECLINED - emitted when the client declines "Would you like to put this product in cart" window
* PUT_IN_CART_FROM_WIDGET - emitted when client [puts a product in cart](#put-in-cart-suggestion) regardless whether the request is initiated by the Streamer or the Client himself. If streamer scanned barcode and sent put in cart request, adds **barcode** param
* APPOINTMENT_CLIENT_ACCEPTED - emitted when the client accepts an appointment in pop-up, adds **appointmentDate** param    
* APPOINTMENT_CLIENT_DECLINED - emitted when the client declines an appointment in pop-up, adds **appointmentDate** param    
* APPOINTMENT_CLIENT_CANCEL - emitted when the client cancels already accepted appointment, adds **appointmentDate** param    
* APPOINTMENT_CLIENT_SHOWED_UP - emitted on first client message after the scheduled date of an accepted appointment had passed (even if the client was late), adds **appointmentDate** param    
* CLIENT_DATA_UPDATED - currently gets emitted on dialog create; always adds **id**; also adds **name**, **email**, **phone**, but they might be empty strings.  
* FOLLOW_LINK - currently only gets emitted on clicking on a PUSH notification (email and sms links do not get tracked); adds **linkType=push**  

##### Event samples
```javascript
// fires every time an element with data-eyezon attribute gets clicked 
eyeZon('on', 'BUTTON_CLICKED', function(data) {
    console.log('BUTTON_CLICKED', data); 
    // title, target and buttonId
})

// fires after a new dialog gets started, once per dialog lifetime    
eyeZon('on', 'DIALOG_CREATED', function(data) {
    console.log('DIALOG_CREATED', data);
    // title, target, buttonId, dialogId, predefinedMessageIndex?, predefinedMessage?, message
    if (data.predefinedMessage) {
        // user clicked on one of the predefined messages
        // data.predefinedMessageIndex - starts at 0
        // data.predefinedMessage 
    } else {
        // data.message - text of the manually typed in message
    }
})

// fires every time a client joins a stream     
eyeZon('on', 'STREAM_JOINED', function(data) {
    console.log('STREAM_JOINED', data)
    // title, target, buttonId, dialogId
})

// fires on each dialog create
eyeZon('on', 'CLIENT_DATA_UPDATED', function(data) {
    data.id; // eyezon client id, always present
    data.name; // might be an empty string if the field isn't required 
    data.email;
    data.phone;
});
```
    


##### Analytics customization
Events can be extended with custom parameters. 
Currently supported parameters are: 
* clientId - id generated by customer's backend or clientId generated by Google Analytics\Yandex Metrics etc
* region - geographical location 

To specify them "setAnalyticsParam" API method has to be called on each page load: 
```javascript
eyeZon('setAnalyticsParam', 'clientId', '123456'); 
eyeZon('setAnalyticsParam', 'region', 'Berlin'); 

// You can also extract '_ga' cookie to get clientId used by Google Analytics  
// or '_ym_uid' cookie to get clientId used by Yandex Metrics  
// But do note they are generated on the Google and Yandex backends, 
// so they aren't immediately available on the the initial page load.
// You might need to duplicate the setAnalyticsParam call in setTimeout. 
// Or just use your own client id, and pass it to trackers with their respective 
// API calls, e.g. ga('set', 'internalId', 'internalIdGeneratedOnBackend') 
```

After this API call subsequent events during the lifetime of the current page will track **clientId** and **region** 

##### Emitting analytics events
To track and match purchases with streamers, host page can emit events. Currently supported events are: 

* PUT_IN_CART  
* CART_SUBMITTED 

To emit an analytics event **emit** API method can be used: 
```javascript
eyeZon('emit', 'PUT_IN_CART', ["target"]); 
eyeZon('emit', 'CART_SUBMITTED', ["target 1", "target 2"], "orderId"); 
```
    
Where **target** is the same target used in data-eyezon-target attribute and **orderId** 
is the order id in customer system. In most cases SKU should be used as **target**.  

The two methods are **independent** of each other. If you only need to track the actual sales, 
emitting CART_SUBMITTED will suffice.  

Emits are processed **only** within **hideEyeAfterMinutes** (24 hours by default) range since the client
 read the last message. So **only pertinent purchases made after a chat or 
 a stream get tracked**. 

Once the conditions are met and the method is called, the widget first searches for an 
active dialog with the same target, 
if not found - searches closed dialogs, 
if not found - uses the first active dialog, 
if not found - uses the first closed dialog. 
This dialog and it's streamer are then deemed accountable for selling the product. 

After the host page emits CART_SUBMITTED for the specified order ID, 
the widget analyzes dialogs and re-emits:

* PRODUCT_SOLD_BY_STREAMER
* PRODUCT_SOLD

##### Emitting analytics events walkthrough
Suppose the client puts a product in cart. To track this action, the host page can emit **PUT_IN_CART** with the API:  
```javascript
eyeZon('emit', 'PUT_IN_CART', ['SKU-1']); 
```
    
Because the client hasn't started a dialog yet, the event gets ignored by the widget.

He then starts a dialog with target *SKU-2*, watches the stream and decides to
put the product in the cart, and the host page calls   
```javascript
eyeZon('emit', 'PUT_IN_CART', ['SKU-2']); 
```

This event is considered pertinent and gets registered by the widget. 
After that the client submits the cart, and 
the host page calls 
```javascript
eyeZon('emit', 'CART_SUBMITTED', ['SKU-1', 'SKU-2'], '14023'); 
```
 
Because there is a dialog with target *SKU-2*, this product is deemed as "sold by the streamer". 
Whereas the *SKU-1* product is deemed "possibly sold by the streamer". The widget then emits events:  

    PRODUCT_SOLD
    {
      ...,
      dialogId: '*last-dialog-id*',
      target: 'SKU-1'
    }

    PRODUCT_SOLD_BY_STREAMER
    {
      ...,
      dialogId: '*pertinent-dialog-id*',
      target: 'SKU-2'
    }


Subsequent **CART_SUBMITTED** emits with the same orderId get ignored. **PUT_IN_CART** emits get registered each time.

24 hours later the client puts another product in the cart and submits it, - all events get ignored, 
because **hideEyeAfterMinutes** has passed since the last message. 

##### Extended analytics events
To track sums, **PUT_IN_CART** and **CART_SUBMITTED** target parameters can be extended with a numeric **sum** property.

```javascript
eyeZon("emit", "PUT_IN_CART", [
  {
    target: "SKU-1",
    sum: 99.99, // price * quantity
  },
]);

eyeZon(
  "emit",
  "CART_SUBMITTED",
  [
    {
      target: "SKU-1",
      sum: 99.99, // price * quantity
    },
    {
      target: "SKU-2",
      sum: 199.99,
    },
  ],
  "14023"
);

// target is either a string or an {target: string, sum: number} object

```



### Put in cart suggestion
The widget UI can show "Put in cart" button. 
The streamer can trigger "Would you like to put this product in cart" window. 
When the client accepts or denies the offer, the corresponding message will be displayed in chat. 

To enable this feature, add PUT_IN_CART_FROM_WIDGET listener  
```javascript
eyeZon('on', 'PUT_IN_CART_FROM_WIDGET', function(data) {
    // use data.target to add the product to cart
    handlePutProductInCart(data.target, 1);  
})
```
    
And add **data-eyezon-put-in-cart** attribute to the trigger element 
```html
<button data-eyezon
        data-eyezon-target="SKU-1"
        data-eyezon-title="Samsung Television"
        data-eyezon-put-in-cart>
    Open chat with "Put in cart" UI enabled
</button>
```
    
If both the conditions are met and the feature is enabled in the button settings, the UI will be enabled for **new dialogs**. 

It is also possible to let the streamer know the client has put the product in cart outside the widget
UI by emitting [PUT_IN_CART](#emitting-analytics-events) analytics event. The cart icon in streamer's app will
get greyed out and STREAMER_PUT_IN_CART_ACCEPTED won't be emitted. 
The dialog has to be active, answered and pertinent. 


### Web View embedding
The widget can be embedded into a WebView of a mobile application.
In most cases **unclosable** param should be set to true.
```javascript
eyeZon('init', {
    businessId: '5e81cf421cb513002ed6f39e',
    region: 'ru-1',
    unclosable: true
});
```

Open the WebView and navigate the user to **https://www.domain.com/?eyezon** (see [Url trigger](#url-triggers) for details)

Window unload event doesn't always fire when destroying a WebView. 
To make sure the client's status is properly set to "offline", the host app needs to fire
leaveDialog and add a small delay before destroying the WebView. 
```javascript
eyeZon('leaveDialog');
```

#### Push Tokens
The host mobile app's FCM or APN tokens can be used to send push notifications when the streamer responds. 
To pass them to the widget, **fcmToken** or **apnToken** and **application** need to be added to query params,
e.g. **https://domain.com/?eyezon&fcmToken={token}&application=AppName** or 
**https://domain.com/?eyezon&apnToken={token}&application=AppName**. Then contact us for further details.

The dialog restoration URL can be built with 
```javascript
data.websiteUrl + '?open=true&buttonId=' + data.buttonId + '&userId=' 
    + data.receiverId + '&dialogId=' + data.dialogId + '&notificationType=push'
```

Note, that after being set the token is only sent once.

So if ?eyezonReset query param is being used, push token must also be appended to the query string indicating it needs to be resent (even if it stays the same). Otherwise, the new client ID won't be associated with it:

https://domain.com/?eyezon&eyezonReset&fcmToken={token}&application=AppName 

### Easter eggs
```javascript
// Disable settings cache (settings get updated on each dialog open or window focus)
window.eyezon_IgnoreSettingsCache = true;

// Disable all events described in Available event types (both emits and server submission)
window.eyezon_IsTestBot = true;

// Disable initial lazy loading  
window.eyeZon_NoExecutionDelay = true;

// Force turn on logging (events disabled by window.eyezon_IsTestBot won't be logged)
window.eyezon_TurnOnLogging = true

// Force event logging, log events as strings, adds CLIENT_ID_KNOWN event (for reading logs from WebView's console on Android\iOS)
window.eyezon_WebViewLogging = true

// Hide notification UI in a WebView and only use host app's push messaing
window.eyezon_WebViewHideNotificationUI = true;
```
